#!/bin/sh
#
# 
# Apusic middleware
#
# Description:	Manages Apusic middleware as Linux-HA resource

#
# Author:	CS2C		
# Support:	support@cs2c.com.cn
# License:	GNU General Public License (GPL)
# Copyright:	(C) 2009 CS2C
# For Apusic version: 7 
#
# See usage() function below for more details...
#
# OCF instance parameters:
#   OCF_RESKEY_APUSIC_HOME=/opt/apusic
#   OCF_RESKEY_APUSIC_domain_path=domains/mydomain
#   OCF_RESKEY_port=6888
#   OCF_RESKEY_sleeptime=10

#######################################################################
# Initialization:
#OCF_ROOT=/usr/lib/ocf
#: ${OCF_FUNCTIONS_DIR=${OCF_ROOT}/lib/heartbeat}
#. ${OCF_FUNCTIONS_DIR}/ocf-shellfuncs
OCF_RESKEY_adminiserverip=${2}
OCF_RESKEY_port=${3}

STATE_OK=0
STATE_WARNING=1
STATE_CRITICAL=2
STATE_UNKNOWN=3
STATE_ERR_ARGS=4

CMD=`basename $0`


#######################################################################

usage() {
  cat <<UEND
	usage: $CMD monitor adminiserverip port

	$CMD manages a DM Database as an HA resource.

	The 'monitor' operation reports whether the database seems to be working

UEND
}

meta_data() {
	cat <<END
<?xml version="1.0"?>
<!DOCTYPE resource-agent SYSTEM "ra-api-1.dtd">
<resource-agent name="Apusic">
<version>1.0</version>

<longdesc lang="en">
Resource script for Apusic. 
It manages Apusic as an HA resource.
</longdesc>
<shortdesc lang="en">Apusic resource agent</shortdesc>

<parameters>

<parameter name="APUSIC_HOME" unique="0" required="1">
<longdesc lang="en">
Directory containing apusic
</longdesc>
<shortdesc lang="en">Apusic installdir</shortdesc>
<content type="string" default="" />
</parameter>

<parameter name="APUSIC_domain_path" unique="0" required="1">
<longdesc lang="en">
Directory containing apusic domain path
</longdesc>
<shortdesc lang="en">Apusic domain path</shortdesc>
<content type="string" default="domains/mydomain" />
</parameter>

<parameter name="port" unique="1" required="1">
<longdesc lang="en">
The Apusic Server Port.
</longdesc>
<shortdesc lang="en">The Apusic Server Port</shortdesc>
<content type="string" default="6888" />
</parameter>

<parameter name="sleeptime" unique="1" required="1">
<longdesc lang="en">
The time start process need to sleep,maybe need change
</longdesc>
<shortdesc lang="en">The time start process need to sleep,maybe need change</shortdesc>
<content type="string" default="10" />
</parameter>

</parameters>

<actions>
<action name="start" timeout="120" />
<action name="stop" timeout="120" />
<action name="status" timeout="60" />
<action name="monitor" depth="0" timeout="30" interval="10" start-delay="10" />
<action name="validate-all" timeout="5" />
<action name="meta-data" timeout="5" />
</actions>
</resource-agent>
END
}

Apusic_monitor() {
	#pid_num_apusic=`ps cax -o pid,command | egrep -w "startapusic" | grep -c startapusic`
         pid_num_apusic=`ps awx -o pid,command | egrep -w "startapusic" | grep -c startapusic`
        #pid_apusic=`ps -o pid  -C startapusic|grep -v PID`
	pid_apusic=`ps -ef | grep startapusic| grep -v grep |awk '{print $2}'`
        if [ ! -z $pid_apusic ];then
            pid_num_java=`ps --ppid $pid_apusic|grep -c java`
	    if [ $pid_num_apusic -ge 1 -a $pid_num_java -ge 1 ]; then
                wget -O- -q -L http://$OCF_RESKEY_adminiserverip:$OCF_RESKEY_port/admin | tr '\012' ' ' | grep -Ei "html" >/dev/null 2>&1
		retval=$?
                if [ $retval -eq 0 ];then         
                   #key-key
                   echo "OK Apusic server is running|state=0"
                   #echo "service_Apusic_ok"
		   exit $STATE_OK 
                else 
                   #key-key
                   echo "CRITICAL Apusic server is error|state=1"
                   #echo "service_Apusic_critical_error "
                   exit $STATE_CRITICAL
                fi
	    else	
                #key-key
 		echo "CRITICAL Apusic server is not running|state=1"
                #echo "service_Apusic_critical_not_running "
		exit $STATE_CRITICAL
	    fi
        else
            #key-key
	    echo "CRITICAL Apusic server is not running|state=1"
            #echo "service_Apusic_critical_not_running "
	    exit $STATE_CRITICAL
        fi
}

#
#       'main' starts here...
#
# What kind of method was invoked?

if
  ( [ $# -ne 3 ] )
then
  usage
  exit $STATE_ERR_ARGS
fi


case "$1" in

  monitor)	Apusic_monitor;;

  *)		usage
		exit $OCF_ERR_UNIMPLEMENTED;;
esac
exit $?
