#!/bin/sh
# WAS6 
#
# Description:  Monitor a Websphere Application Server 7
#
#
# Author:       Ru Xiang Min
# Support:      linux-ha@lists.linux-ha.org
# License:      GNU General Public License (GPL)
# Copyright:    (C) 2006 International Business Machines China, Ltd., Inc.
#
#
# An example usage in /etc/ha.d/haresources:
#       node1  10.0.0.170 WAS::/opt/IBM/WebSphere/AppServer/profiles/default/config/cells/Node01Cell/nodes/Node01/serverindex.xml
#
# See usage() function below for more details...
#
#         OCF parameters are as below:
#               OCF_RESKEY_profile
#                 (WAS profile name, used for the single server edition of WAS6)

#######################################################################
# Initialization:

#: ${OCF_FUNCTIONS_DIR=${OCF_ROOT}/lib/heartbeat}
#. ${OCF_FUNCTIONS_DIR}/ocf-shellfuncs

OCF_RESKEY_adminiserverip=${2}
OCF_RESKEY_port=${3}

STATE_OK=0
STATE_WARNING=1
STATE_CRITICAL=2
STATE_UNKNOWN=3
STATE_ERR_ARGS=4

#######################################################################

WAS_DIR=/opt/IBM/WebSphere/AppServer
if
  [ ! -d $WAS_DIR ]
then
  WAS_DIR=/usr/IBM/WebSphere/AppServer
fi
STARTTIME=300   #       5 minutes
#
#
CMD=`basename $0`
#
#       Print usage message
#
usage() {
  cat <<-END
        usage: $CMD monitor adminiserverip port 


        $CMD Monitor a Websphere Application Server 7 

        The 'monitor' operation reports whether the WAS7 seems to be working

	END
}

#
#       Reformat the XML document in a sort of canonical form
#       if we can.  If we don't have xmllint, we just cat it out
#       and hope for the best ;-)
#
xmlcat() {
echo "xmlcat"
}

#
#This is a bit skanky, but it works anyway...
#
# It's not really skanky if we can find xmllint on the system, because it
# reformats tags so they are all on one line, which is all we we need...
#
#
# Get the numbers of the ports WAS should be listening on...
#
# If we don't have xmllint around, then the applicationserver and the
# port= specification have to be on the same line in the XML config file.
#
GetWASPorts() {
echo "GetWASPorts"
}

#
#       We assume that the seventh port listed in the serverindex.xml
#       is the one we should run servlet/snoop on.
#
GetWASSnoopPort() {
echo "GetWASSnoopPort"
}

#
#       Return information on the processname/id for the WAS ports
#
#       pid/java        is the expected output.  Several lines, one per port...
#
#
WASPortInfo() {
echo "WASPortInfo"
}

#
#       Return the number of WAS ports which are open
#
CheckWASPortsInUse() {
  echo "CheckWASPortsInUse" 
}

#
#       Return the pid(s) of the processes that have WAS ports open
#
WASPIDs() {
  echo "WASPIDs" 
}

#
#       The version of ps that returns all processes and their (long) args
#       It's only used by WAS_procs, which isn't used for anything ;-)
#
ps_long() {
  echo "ps_long" 
}


#
#       The total set of WAS processes (single server only)
#
WAS_procs() {
  echo "WAS_procs" 
}



#
# methods: What methods/operations do we support?
#
WAS_methods() {
  echo "WAS_methods" 
}

#
#       Return WAS status (silently)
#
WAS_status() {
  echo "WAS_status" 
}

#
#       Report on WAS status to stdout...
#
WAS_report_status() {
  echo "WAS_report_status" 
}

#
#       Monitor WAS - does it really seem to be working?
#
#       For this we invoke the snoop applet via wget.
#
#       This is actually faster than WAS_status above...
#
WAS_monitor() {
    chk_admini=`ps awx | grep WebSphere/AppServer|grep -v 'grep'|wc -l`
    if [ $chk_admini -ge 1 ]; then
        sleep 2

        wget -O- -q -L http://$OCF_RESKEY_adminiserverip:$OCF_RESKEY_port/ibm/console | grep -Ei "html" >/dev/null 2>&1
        rc=$?
        if [ $rc -eq 0 ]; then
            #key-key
            echo "OK Login to console is OK !|state=0"
            #echo "service_was_ok"
            exit $STATE_OK
        else
            #key-key
            echo "CRITICAL WAS: $1: wget failure: $output|state=1"
            #echo "service_was_critical_wget_fail#$output"
            exit $STATE_CRITICAL
        fi
    else
        #key-key
        echo "CRITICAL WAS is not running|state=1"
        #echo "service_was_critical_not_running"
        exit $STATE_CRITICAL
    fi
}

#
#       Start WAS instance
#
WAS_start() {
      echo "WAS_start"
}

#
#       Wait for WAS to actually start up.
#
#       It seems to take between 30 and 60 seconds for it to
#       start up on a trivial WAS instance.
#
WAS_wait_4_start() {
      echo "WAS_wait_4_start"
}


#
#       Shut down WAS
#
WAS_stop() {
      echo "WAS_stop"
}

#
#       Check if the port is valid
#
CheckPort() {
      echo "CheckPort"
}

WAS_validate_all() {
      echo "WAS_validate_all"
}
#
#       'main' starts here...
#

if
  ( [ $# -ne 3 ] )
then
  usage
  exit $STATE_ERR_ARGS 
fi

# These operations don't require OCF instance parameters to be set
case "$1" in

  meta-data)    meta_data
                exit $OCF_SUCCESS;;

  usage)        usage
                exit $OCF_SUCCESS;;

  methods)      WAS_methods
                exit $?;;
  *);;
esac

# What kind of method was invoked?
case "$1" in

  start)        WAS_start $arg
                exit $?;;

  stop)         WAS_stop $arg
                exit $?;;

  status)       WAS_report_status $arg
                exit $?;;

  monitor)      WAS_monitor
                exit $?;;

  validate-all) WAS_validate_all $arg
                exit $?;;

  *)            usage
                exit $OCF_ERR_UNIMPLEMENTED;;
esac

