#! /bin/sh
#
# 
# Weblogic
#
# Description:	Manages an Weblogic 10.3 as a NeoShineHA resource
#
# See usage() function below for more details...
#
# OCF instance parameters:
OCF_RESKEY_beahome=${2}
#OCF_RESKEY_domainname=${3}
OCF_RESKEY_adminiserverip=${3}
OCF_RESKEY_port=${4}
#OCF_RESKEY_sleeptime=${6}

CMD=`basename $0`

STATE_OK=0
STATE_WARNING=1
STATE_CRITICAL=2
STATE_UNKNOWN=3
STATE_ERR_ARGS=4

# Initialization:

#. ${OCF_ROOT}/resource.d/heartbeat/.ocf-shellfuncs
# Initialization:
#OCF_ROOT=/usr/lib/ocf
#: ${OCF_FUNCTIONS_DIR=${OCF_ROOT}/lib/heartbeat}
#. ${OCF_FUNCTIONS_DIR}/ocf-shellfuncs
#######################################################################

usage() {
  methods=`weblogic_methods`
  methods=`echo $methods | tr ' ' '|'`
  cat <<-!
	usage: $CMD {$methods}

	$CMD manages an Weblogic 10.3 instance as an HA resource.

	The 'monitor' operation reports whether the weblogic seems to be working
	The 'validate-all' operation reports whether the parameters are valid
	The 'methods' operation reports on the methods $CMD supports
	
	usage: $CMD monitor beahome adminiserverip port
	!

}

#meta_data() {
#	cat <<END
#<?xml version="1.0"?>
#<!DOCTYPE resource-agent SYSTEM "ra-api-1.dtd">
#<resource-agent name="weblogic">
#<version>1.0</version>

#<longdesc lang="en">
#Resource script for Weblogic. Manages an Weblogic 9.2 instance
#as an HA resource.
#</longdesc>
#<shortdesc lang="en">weblogic resource agent</shortdesc>

#<parameters>

#<parameter name="beahome" unique="1" required="1">
#<longdesc lang="en">
#The weblogic home directory.
#</longdesc>
#<shortdesc lang="en">beahome</shortdesc>
#<content type="string" default="" />
#</parameter>
#
#<parameter name="domainname" unique="1" required="1">
#<longdesc lang="en">
#The Domanin Name.
#</longdesc>
#<shortdesc lang="en">domainname</shortdesc>
#<content type="string" default="" />
#</parameter>

#<parameter name="adminiserverip" unique="1" required="1">
#<longdesc lang="en">
#The Admini Server IP.
#</longdesc>
#<shortdesc lang="en">Admini Server IP</shortdesc>
#<content type="string" default="" />
#</parameter>

#<parameter name="port" unique="1" required="1">
#<longdesc lang="en">
#The Admini Server Port.
#</longdesc>
#<shortdesc lang="en">The Admini Server Port</shortdesc>
#<content type="string" default="" />
#</parameter>

#<parameter name="sleeptime" unique="1" required="1">
#<longdesc lang="en">
#The time start process need to sleep,maybe need change
#</longdesc>
#<shortdesc lang="en">The time start process need to sleep,maybe need change</shortdesc>
#<content type="string" default="30" />
#</parameter>


#</parameters>

#<actions>
#<action name="monitor" depth="0" timeout="30" interval="60" start-delay="10" />
#<action name="validate-all" timeout="5" />
#<action name="meta-data" timeout="5" />
#</actions>
#</resource-agent>
#END
#}


#
# methods: What methods/operations do we support?
#
weblogic_methods() {
  cat <<-!
	monitor
	methods
	usage
	!
}

weblogic_monitor() {
    chk_admini=`ps awx | grep startWebLogic.sh |grep -v 'grep'|wc -l`
  if [ $chk_admini -ge 1 ]; then
    sleep 2
    wget -O- -q -L http://$OCF_RESKEY_adminiserverip:$OCF_RESKEY_port/console | tr '\012' ' ' | grep -Ei "html" >/dev/null 2>&1
    chk_wget=$?
    if [ $chk_wget -eq 0 ]; then
	#key-key
	echo "OK Weblogic is running|state=0"
	#echo "service_weblogic_ok"
	exit $STATE_OK
    else
	#key-key
	echo "CRITICAL Weblogic is error|state=1 "
	#echo "service_weblogic_critical_error "
	exit $STATE_CRITICAL
    fi
  else
    #key-key
    echo "CRITICAL Weblogic is not running|state=1"
    #echo "service_weblogic_critical_not_running "
    exit $STATE_CRITICAL
  fi
}

weblogic_validate() {
 if [ ! -d $OCF_RESKEY_beahome ]; then
        echo "Weblogic InstallDir $OCF_RESKEY_beahome dosen't exist";
        exit $STATE_ERR_ARGS;
 fi
}

#
#       'main' starts here...
#

if
  ( [ $# -ne 4 ] )
then
  usage
  exit $STATE_ERR_ARGS 
fi

case "$1" in

  monitor)	weblogic_validate;
		weblogic_monitor;;

  validate-all)	weblogic_validate; exit $OCF_SUCCESS;;

# meta-data) meta_data; exit $OCF_SUCCESS;;

  *)		usage
		exit $OCF_ERR_UNIMPLEMENTED;;
esac
exit $?
